structure T = Tokens
structure I = Interface
structure A = Absyn

open I

type pos = I.pos
type svalue = T.svalue
type ('a,'b) token = ('a,'b) T.token
type lexresult = (svalue,pos) token list * string

fun eof() = ([T.EOF(!line,!line)],"")

fun CN(s:string) = ([T.CN(A.ConS s,!line,!line)],s)
fun RNs(s:string) = ([T.RNs(A.ConR s,!line,!line)],s)
fun RNp(s:string) = ([T.RNp(A.ConR s,!line,!line)],s)
fun PN(s:string) = ([T.PN(A.ConS s,!line,!line)],s)
fun IVs(s:string) = ([T.IVs(A.ConS s,!line,!line)],s)
fun IVp(s:string) = ([T.IVp(A.ConS s,!line,!line)],s)
fun TVs(s:string) = ([T.TVs(A.ConR s,!line,!line)],s)
fun TVp(s:string) = ([T.TVp(A.ConR s,!line,!line)],s)
fun CAs(s:string) = ([T.CAs(A.ConS s,!line,!line)],s)
fun CAp(s:string) = ([T.CAp(A.ConS s,!line,!line)],s)
fun IAComp(s:string) = ([T.IAComp(A.ConR s,!line,!line)],s)
fun P(s:string) = ([T.P(A.ConR s,!line,!line)],s)

%%  
%header (functor NL_LexFun(structure Absyn: ABSYN
                           structure Tokens: NL__TOKENS
                              sharing type Absyn.set = Tokens.set 
                           structure Interface: INTERFACE) : LEXER);
%full 

whitespace=[\t\ ]*;
%%
<INITIAL>{whitespace} => (lex());
\n         => (next_line(); lex());
"."        => ([T.PUNKT(!line,!line)],yytext);
"ist"      => ([T.CopVs(!line,!line)],yytext);
"sind"      => ([T.CopVp(!line,!line)],yytext);
"von"    => ([T.von(!line,!line)],yytext);
("der" | "die" | "das" | "den")   
           => ([T.Det(!line,!line)],yytext);
("Ein" | "Eine" |"einen" | "ein" | "eine" | "eines")   
           => ([T.EQ(!line,!line)],yytext);
("Kein" | "Keine" | "keinen" | "kein" | "keine" | "keines")   
           => ([T.NQ(!line,!line)],yytext);
("Alle" | "alle" | "Jeder" | "jeder" 
                 | "Jede" | "jede" | "Jeden" | "jeden")   
           => ([T.UQ(!line,!line)],yytext);
"Anna"     => (PN "Anna");
"Dieter"   => (PN "Dieter");
"Emil"     => (PN "Emil");
"Maria"    => (PN "Maria");
"Waldi"    => (PN "Waldi");
"Rom"   => (PN "Rom");
"Madrid"   => (PN "Madrid");
"Italien"   => (PN "Italien");
"Spanien"   => (PN "Spanien");
"Mann"     => (CN "Mann");
"Frau"     => (CN "Frau");
("Hund" | "Hunde")
           => (CN "Hund");
("Berg" | "Berge")     
           => (CN "Berg");
("Student" | "Studenten") 
           => (CN "Student");
"Nachbar"  => (RNs "Nachbar");  
"Bruder"   => (RNs "Bruder");   
"Schwester"  => (RNs "Schwester"); 
"Hauptstadt"   => (RNs "Hauptstadt");
"Stadt"    => (CN "Stadt");
"gesund"   => (CAs "gesund");
("braun" | "braune" | "brauner")
           => (CAs "braun");
"braune"    => (CAp "braun");
("toll" | "tolle" | "toller")
           => (CAs "toll");
"tolle"    => (CAp "toll");
("netter" | "nette" | "nettes")
           => (CAs "nett");
("nette")  => (CAp "nette");
"schöner als"   => (IAComp "schöner als");
"älter als"   => (IAComp "älter als");
("in") => (P "in");
"arbeiten" => (IVp "arbeiten");
"arbeitet" => (IVs "arbeiten");
"kennen"   => (TVp "kennen");
"kennt"    => (TVs "kennen");
"lieben"   => (TVp "lieben");
"liebt"    => (TVs "lieben");
.          => (error ("ignoring illegal character " ^ yytext,
                      !line,!line); lex());


