signature SET = (* Naive implementation of small sets via lists. *)
  sig
    type 'a set
    val empty : 'a set
    val add : ''a -> ''a set -> ''a set
    val difference : ''a set -> ''a set -> ''a set
    val drop : ''a -> ''a set -> ''a set
    val equal : ''a set -> ''a set -> bool
    val intersection : ''a set -> ''a set -> ''a set
    val intersects : ''a set -> ''a set -> bool
    val list : 'a set -> 'a list
    val map : ('a -> ''b) -> 'a set -> ''b set
    val member : ''a -> ''a set -> bool
    val mk : ''a list -> ''a set
    val select : 'a set -> ('a -> bool) -> 'a set
    val size : 'a set -> int
    val singleton : 'a -> 'a set 
    val subset : ''a set -> ''a set -> bool
    val union : ''a set -> ''a set -> ''a set
    val Union : ''a set list -> ''a set
  end

structure Set : SET =
    struct
        datatype 'a set = Set of 'a list

        val empty = Set []

        fun member x (Set []) = false
          | member x (Set (y::ys)) = (x = y) orelse (member x (Set ys))
        and add x (Set S) = 
            let fun addLst x [] = [x]
                  | addLst x (u::U) = if x=u then (u::U) 
                                      else u::(addLst x U)
            in Set (addLst x S) end
        and intersects (Set []) Q = false
          | intersects (Set (p::P)) Q = 
            (member p Q) orelse (intersects (Set P) Q)
        and intersection (Set []) Q = Set []
          | intersection (Set (p::P)) Q = 
            if member p Q then add p (intersection (Set P) Q) 
            else intersection (Set P) Q
        and union (Set []) U = U
          | union (Set (x::Q)) U = (* x is not in Q! *)
            union (Set Q) (add x U)
        and Union sets = 
            let 
                fun bigU [] (Set U) = (Set U)
                  | bigU (Q::Qs) U = bigU Qs (union Q U)
            in 
                bigU sets empty
            end
        and drop e (Set S) =
            let fun subtr [] e = []
                  | subtr (s::S) e = if s=e then S (* s not in S! *)
                                     else (s::(subtr S e))
            in Set (subtr S e) end
        and difference (Set S) (Set T) =
            let fun diff (Set S) [] = S  
                  | diff (Set S) (e::E) = diff (drop e (Set S)) E
            in Set (diff (Set S) T) end

        fun size (Set S) = List.length S
        and mk L = Union (map (fn x => Set [x]) L) 
        and list (Set S) = S
        and select (Set S) f =
            let fun selct [] f = []
                  | selct (s::S) f = if f s then (s::(selct S f))
                                     else (selct S f)
            in Set (selct S f) end

        fun subset S T = 
            let fun f x = member x T
                val U = select S f
            in size U = size S end
        and equal S T = (size S = size T) andalso (subset S T)

        fun map f (Set S) = mk (List.map f S)
        fun singleton e = (Set [e])  
    end

