structure T = Tokens
structure I = Interface
structure A = Absyn

open I

type pos = I.pos
type svalue = T.svalue
type ('a,'b) token = ('a,'b) T.token
type lexresult = (svalue,pos) token list * string

fun eof() = ([T.EOF(!line,!line)],"")

fun PN(s:string) = ([T.PN(A.ConS s,!line,!line)],s)
fun CNn s t = ([T.CNn(A.ConS s,!line,!line)],t)
fun CNg s t = ([T.CNg(A.ConS s,!line,!line)],t)
fun CNd s t = ([T.CNd(A.ConS s,!line,!line)],t)
fun CNa s t = ([T.CNa(A.ConS s,!line,!line)],t)
fun RNsn s t = ([T.RNsn(A.ConR s,!line,!line)],t)
fun RNsg s t = ([T.RNsg(A.ConR s,!line,!line)],t)
fun RNsd s t = ([T.RNsd(A.ConR s,!line,!line)],t)
fun RNsa s t = ([T.RNsa(A.ConR s,!line,!line)],t)
fun RNpn s t = ([T.RNpn(A.ConR s,!line,!line)],t)
fun RNpg s t = ([T.RNpg(A.ConR s,!line,!line)],t)
fun RNpd s t = ([T.RNpd(A.ConR s,!line,!line)],t)
fun RNpa s t = ([T.RNpa(A.ConR s,!line,!line)],t)
fun IVs(s:string) = ([T.IVs(A.ConS s,!line,!line)],s)
fun IVp(s:string) = ([T.IVp(A.ConS s,!line,!line)],s)
fun TVs(s:string) = ([T.TVs(A.ConR s,!line,!line)],s)
fun TVp(s:string) = ([T.TVp(A.ConR s,!line,!line)],s)
fun CAsn(s:string) = ([T.CAsn(A.ConS s,!line,!line)],s)
fun CAsg(s:string) = ([T.CAsg(A.ConS s,!line,!line)],s)
fun CAsd(s:string) = ([T.CAsd(A.ConS s,!line,!line)],s)
fun CAsa(s:string) = ([T.CAsa(A.ConS s,!line,!line)],s)
fun CApn(s:string) = ([T.CApn(A.ConS s,!line,!line)],s)
fun CApg(s:string) = ([T.CApg(A.ConS s,!line,!line)],s)
fun CApd(s:string) = ([T.CApd(A.ConS s,!line,!line)],s)
fun CApa(s:string) = ([T.CApa(A.ConS s,!line,!line)],s)
fun IAComp(s:string) = ([T.IAComp(A.ConR s,!line,!line)],s)
fun P(s:string) = ([T.P(A.ConR s,!line,!line)],s)

%%  
%header (functor NL_LexFun(structure Absyn: ABSYN
                           structure Tokens: NL__TOKENS
                              sharing type Absyn.set = Tokens.set 
                           structure Interface: INTERFACE) : LEXER);
%full 

whitespace=[\t\ ]*;
%%
<INITIAL>{whitespace} => (lex());
\n         => (next_line(); lex());
"."        => ([T.PUNKT(!line,!line)],yytext);
"ist"      => ([T.CopVs(!line,!line)],yytext);
"sind"      => ([T.CopVp(!line,!line)],yytext);
"von"    => ([T.von(!line,!line)],yytext);
("der" | "die" | "das")   
           => ([T.Detn(!line,!line)],yytext);   
("des" | "der" | "des")     
           => ([T.Detg(!line,!line)],yytext);   
("dem" | "der" | "dem")   
           => ([T.Detd(!line,!line)],yytext);   
("den" | "die" | "das")   
           => ([T.Deta(!line,!line)],yytext);
("Ein" | "Eine" | "ein" | "eine") 
        => ([T.EQn(!line,!line)],yytext); 
("Eines" | "Einer" | "eines" | "einer") 
        => ([T.EQg(!line,!line)],yytext); 
("Einem" | "Einer" | "einem" | "einer") 
        => ([T.EQd(!line,!line)],yytext);
("Einen" | "Eine" | "Ein" |"einen" | "eine" | "ein") 
        => ([T.EQa(!line,!line)],yytext);
("Kein" | "Keine" | "kein" | "keine") 
        => ([T.NQn(!line,!line)],yytext);
("Keines" | "Keiner" | "keines" | "keiner") 
        => ([T.NQg(!line,!line)],yytext);  
("Keinem" | "Keiner" | "keinem" | "keiner") 
        => ([T.NQd(!line,!line)],yytext); 
("Keinen" | "Keine" | "Kein" | "keinen" | "keine" | "kein") 
        => ([T.NQa(!line,!line)],yytext); 
("Alle" | "alle" | "Jeder" | "Jede" | "Jedes" | "jeder" | "jede" | "jedes") 
        => ([T.UQn(!line,!line)],yytext);
("Aller" | "aller" | "Jedes" | "Jeder" | "jedes" | "jeder")
        => ([T.UQg(!line,!line)],yytext); 
("Allen" | "allen" | "Jedem" | "Jeder" | "jedem" | "jeder")
        => ([T.UQd(!line,!line)],yytext);
("Alle" | "alle" | "Jeden" | "Jede" | "Jedes" | "jeder" | "jede" | "jedes")
        => ([T.UQa(!line,!line)],yytext);
"Anna"     => (PN "Anna");
"Dieter"   => (PN "Dieter");
"Emil"     => (PN "Emil");
"Maria"    => (PN "Maria");
"Waldi"    => (PN "Waldi");
"Rom"   => (PN "Rom");
"Madrid"   => (PN "Madrid");
"Italien"   => (PN "Italien");
"Spanien"   => (PN "Spanien");
"Mann"     => (CNn "Mann" yytext);
"Mannes"   => (CNg "Mann" yytext);
"Mann"     => (CNd "Mann" yytext);
"Mann"     => (CNa "Mann" yytext);
"Frau"     => (CNn "Frau" yytext);
"Frau"     => (CNg "Frau" yytext);
"Frau"     => (CNd "Frau" yytext);
"Frau"     => (CNa "Frau" yytext);
"Hund"     => (CNn "Hund" yytext);
"Hundes"     => (CNg "Hund" yytext);
"Hund"     => (CNd "Hund" yytext);
"Hund"     => (CNa "Hund" yytext);
"Student"   => (CNn "Student" yytext);
"Studenten" => (CNg "Student" yytext);
"Studenten" => (CNd "Student" yytext);
"Studenten" => (CNa "Student" yytext);
"Bruder"   => (RNsn "Bruder" yytext);   
"Bruders"   => (RNsg "Bruder" yytext); 
"Bruder"   => (RNsd "Bruder" yytext);   
"Bruder"   => (RNsa "Bruder" yytext);   
"Schwester"  => (RNsn "Schwester" yytext); 
"Schwester"  => (RNsg "Schwester" yytext); 
"Schwester"  => (RNsd "Schwester" yytext); 
"Schwester"  => (RNsa "Schwester" yytext);
"Nachbar"  => (RNsn "Nachbar" yytext);   
"Nachbars"  => (RNsg "Nachbar" yytext);   
"Nachbarn"  => (RNsd "Nachbar" yytext);   
"Nachbarn"  => (RNsa "Nachbar" yytext);   
"Hauptstadt"   => (RNsn "Hauptstadt" yytext);
"Hauptstadt"   => (RNsg "Hauptstadt" yytext);
"Hauptstadt"   => (RNsd "Hauptstadt" yytext);
"Hauptstadt"   => (RNsa "Hauptstadt" yytext);   
"gesund"   => (CAsn "gesund");
("brauner" | "braunes" | "braune")
             => (CAsn "braun");
("braunen" | "brauner")
             => (CAsg "braun");
("braunem" | "brauner")
             => (CAsd "braun");
("braunen" | "braune" | "braunes")
             => (CAsa "braun");
"braune"    => (CApn "braun");
"brauner"    => (CApg "braun");
"braunen"    => (CApd "braun");
"braune"    => (CApa "braun");
("toller" | "tolle" | "tolles")
             => (CAsn "toll");
("tollen" | "toller")     
             => (CAsg "toll");
("tollem" | "toller")
             => (CAsd "toll");
("tollen" | "tolle" | "tolles")
             => (CAsa "toll");
"tolle"      => (CApn "toll");
"toller"      => (CApg "toll");
"tollen"      => (CApd "toll");
"tolle"      => (CApa "toll");
("netter" | "nette" | "nettes")
            => (CAsn "nett");
("netten" | "netter")
            => (CAsg "nett");
("nettem" | "netter")
            => (CAsd "nett");
("netten" | "nette" | "nettes")
            => (CAsa "nett");
"nette"    => (CApn "nett");
"netter"    => (CApg "nett");
"netten"    => (CApd "nett");
"nette"    => (CApa "nett");
"schöner als"   => (IAComp "schöner als");
"älter als"   => (IAComp "älter als");
("in") => (P "in");
"arbeiten" => (IVp "arbeiten");
"arbeitet" => (IVs "arbeiten");
"kennen"     => (TVp "kennen");
"kennt"      => (TVs "kennen");
"lieben"     => (TVp "lieben"); 
"liebt"      => (TVs "lieben");
"tanzen"     => (TVp "tanzen"); 
"tanzt"      => (TVs "tanzen");
.          => (error ("ignoring illegal character " ^ yytext,
                      !line,!line); lex());


