structure PeirceTerm =
   struct
       datatype set = ConS of string 
                     | SumS of set * set
                     | ZeroS
                     | ProdS of set * set
                     | UnitS 
                     | CompS of set
                     | PreIm of rel * set   (* R : B *)
       and      rel = ConR of string
                     | SumR of rel * rel
                     | ZeroR
                     | ProdR of rel * rel
                     | UnitR
                     | CompR of rel
                     | Id of set
                     | Minor of rel
                     | Tc of rel
                     | Prod of rel * rel
                     | Conv of rel
                     | Cylin of set
       and      tv  = E of set
                     | N of set
                     | U of set
                     | ER of rel
                     | UR of rel
                     | NR of rel
                     | And of tv * tv
                     | Or  of tv * tv
                     | Neg of tv

    val I = Id UnitS

    fun StoString ZeroS       = "0" 
      | StoString UnitS       = "1" 
      | StoString (ConS(w))   = w
      | StoString (SumS(A,B)) = 
        "(" ^ (StoString A) ^ " + " ^ (StoString B) ^ ")"
      | StoString (ProdS(A,B)) = 
        "(" ^ (StoString A) ^ " . " ^ (StoString B) ^ ")"
      | StoString (CompS(A))   = "-" ^ (StoString A) 
      | StoString (PreIm(R,A)) = 
        "(" ^ (RtoString R) ^ " : " ^ (StoString A) ^ ")"
    and RtoString (ConR(r))    = r
      | RtoString UnitR        = " 1' " 
      | RtoString ZeroR        = " 0' " 
      | RtoString (SumR(R,S))  = 
        "(" ^ (RtoString R) ^ " + " ^ (RtoString S) ^ ")"
      | RtoString (ProdR(Cylin A,Conv(Cylin B))) = 
        "(" ^ (StoString A) ^ " x " ^ (StoString B) ^ ")"                   
      | RtoString (ProdR(R,S)) = 
        "(" ^ (RtoString R) ^ " . " ^ (RtoString S) ^ ")"
      | RtoString (CompR(R))   = "-" ^ (RtoString R) 
      | RtoString (Minor(R))   = "minor" ^ (RtoString R)
      | RtoString (Tc(R))      = "tc(" ^ (RtoString R) ^ ")"
      | RtoString (Id(A))      = "Id(" ^ (StoString A) ^ ")"
      | RtoString (Prod(R,S))  = 
        "(" ^ (RtoString R) ^ " ; " ^ (RtoString S) ^ ")"
      | RtoString (Conv(R))    = "~" ^ (RtoString R) 
      | RtoString (Cylin(A))   = "("^(StoString A) ^ " x 1)" 
    and TtoString (E A) = "0 < " ^ (StoString A) 
      | TtoString (U A) = "1 = " ^ (StoString A) 
      | TtoString (N A) = "0 = " ^ (StoString A) 
      | TtoString (ER A) = "0' < " ^ (RtoString A) 
      | TtoString (UR A) = "1' = " ^ (RtoString A) 
      | TtoString (NR A) = "0' = " ^ (RtoString A) 
      | TtoString (Neg A) = "-(" ^ (TtoString A) ^ ")"
      | TtoString (And(A,B)) = "(" ^ (TtoString A) ^ " /\\ " 
                                ^ (TtoString B) ^ ")" 
      | TtoString (Or(A,B)) = "(" ^ (TtoString A) ^ " \\/ " 
                                       ^ (TtoString B) ^ ")" 
   end

signature UNIVERSE = 
   sig
       eqtype element 
       val elements : element list 
       val ConS : string -> element list
       val ConR : string -> (element * element) list
       exception NoValue 
   end

signature PEIRCEfull = 
   sig 
       include PEIRCE
       type element 
   end                     
    
functor Peirce(U : UNIVERSE) : PEIRCEfull = 
   struct
     type element = U.element
     fun ConS s = Set.mk (U.ConS s)
     fun ConR s = Set.mk (U.ConR s)

     type set = element Set.set

     val UnitS = Set.mk U.elements
     val ZeroS = Set.empty
     fun SumS(A,B) = Set.union A B
     fun ProdS(A,B) = Set.intersection A B
     fun CompS(A) = Set.difference UnitS A

     type rel = (element * element) Set.set

     fun Id(A) = Set.map (fn x => (x,x)) A
     val I = Id(UnitS)
     fun Cart(A,B) = 
         let 
             fun f x = List.map (fn y => (x,y)) (Set.list B)
         in 
             Set.mk (List.foldr (op @) nil (List.map f (Set.list A)))
         end
     val UnitR:rel = Cart(UnitS,UnitS)
     fun Cylin(A) = Cart(A,UnitS)
     val ZeroR = Set.empty
     fun SumR(R,S) = Set.union R S
     fun ProdR(R,S) = Set.intersection R S
     fun CompR(R) = Set.difference UnitR R
     fun Conv(R) = Set.map (fn (x,y) => (y,x)) R

     fun Prod(R,S) = 
         let 
             fun f(a,b) = Set.map (fn (x,c) => (a,c)) 
                                  (Set.select S (fn (x,c) => x = b))
         in
             Set.Union (List.map f (Set.list R))
         end

     fun PreIm(R,B) =
         Set.map #1 (Set.select R (fn (a,b) => Set.member b B))
     fun Minor(R) = 
         let 
             val domain = PreIm(R,UnitS)
             val range = PreIm(Conv R,UnitS)
             val S = ProdR(R,Cart(range,domain))
         in 
             if Set.size(R) = Set.size(S) then ZeroR else 
             SumR(ProdR(R,Cart(CompS range,range)),Minor(S))
         end
     fun Tc(R) = 
         let 
             val T = SumR(Prod(R,R),R) 
         in
             if Set.size(T) = Set.size(R) then R else Tc(T) 
         end

     datatype tv = datatype bool

     fun E(A) = (Set.size A > 0)
     fun N(A) = (Set.size A = 0)
     fun U(A) = ((Set.size A) = (Set.size UnitS))

     fun ER(A) = (Set.size A > 0)
     fun UR(A) = ((Set.size A) = (Set.size UnitR))
     fun NR(A) = (Set.size A = 0)

     fun And(A,B) = A andalso B
     fun Or(A,B)  = A orelse B
     fun Neg(A)  = not A
   end 

