structure T = Tokens
structure I = Interface
structure A = Absyn

open I

type pos = I.pos
type svalue = T.svalue
type ('a,'b) token = ('a,'b) T.token
type lexresult = (svalue,pos) token list * string

fun eof() = ([T.EOF(!line,!line)],"")

fun CN(s:string) = ([T.CN(A.ConS s,!line,!line)],s)
fun RN(s:string) = ([T.RN(A.ConR s,!line,!line)],s)
fun PN(s:string) = ([T.PN(A.ConS s,!line,!line)],s)
fun IV(s:string) = ([T.IV(A.ConS s,!line,!line)],s)
fun TV(s:string) = ([T.TV(A.ConR s,!line,!line)],s)
fun CA(s:string) = ([T.CA(A.ConS s,!line,!line)],s)
fun IA(s:string) = ([T.IA(A.ConS s,!line,!line)],s)
fun IAComp(s:string) = ([T.IAComp(A.ConR s,!line,!line)],s)
fun P(s:string) = ([T.P(A.ConR s,!line,!line)],s)

%%  
%header (functor NL_LexFun(structure Absyn: ABSYN
                           structure Tokens: NL__TOKENS
                              sharing type Absyn.set = Tokens.set 
                           structure Interface: INTERFACE) : LEXER);
%full 

whitespace=[\t\ ]*;
%%
<INITIAL>{whitespace} => (lex());
\n         => (next_line(); lex());
"."        => ([T.PUNKT(!line,!line)],yytext);
"ist"      => ([T.CopV(!line,!line)],yytext);
"von"    => ([T.von(!line,!line)],yytext);
("der" | "die" | "das" | "den")   
           => ([T.Det(!line,!line)],yytext);
("Ein" | "Eine" |"einen" | "ein" | "eine" | "eines")   
           => ([T.EQ(!line,!line)],yytext);
("Kein" | "Keine" | "keinen" | "kein" | "keine" | "keines")   
           => ([T.NQ(!line,!line)],yytext);
("Alle" | "alle" | "Jeder" | "jeder" 
                 | "Jede" | "jede" | "Jeden" | "jeden")   
           => ([T.UQ(!line,!line)],yytext);
"Anna"     => (PN "Anna");
"Dieter"   => (PN "Dieter");
"Emil"     => (PN "Emil");
"Maria"    => (PN "Maria");
"Waldi"    => (PN "Waldi");
"Rom"   => (PN "Rom");
"Madrid"   => (PN "Madrid");
"Italien"   => (PN "Italien");
"Spanien"   => (PN "Spanien");
"Mann"     => (CN "Mann");
"Frau"     => (CN "Frau");
("Hund" | "Hunde")
           => (CN "Hund");
"Berg"     => (CN "Berg");
("Student" | "Studenten")
           => (CN "Student");
"Nachbar"  => (RN "Nachbar");  
"Bruder"   => (RN "Bruder");   
"Schwester"  => (RN "Schwester"); 
"Hauptstadt"   => (RN "Hauptstadt");
"Stadt"    => (CN "Stadt");
"gesund"   => (CA "gesund");
("braun" | "braune" | "brauner")  => (CA "braun");
("toll" | "tolle" | "toller")     => (CA "toll");
("nett" | "nette" | "netter" | "netten")
          => (CA "nett");
"schön"   => (IA "schön");
"schöner als"   => (IAComp "schöner als");
"älter als"   => (IAComp "älter als");
("in") => (P "in");
("arbeiten" | "arbeitet") => (IV "arbeiten");
("kennen" | "kennt")      => (TV "kennen");
("lieben" | "liebt")      => (TV "lieben");
.          => (error ("ignoring illegal character " ^ yytext,
                      !line,!line); lex());


