functor NL_LexFun(structure Absyn: ABSYN
                           structure Tokens: NL__TOKENS
                              sharing type Absyn.set = Tokens.set 
                           structure Interface: INTERFACE) : LEXER  = struct

    structure yyInput : sig

        type stream
	val mkStream : (int -> string) -> stream
	val fromStream : TextIO.StreamIO.instream -> stream
	val getc : stream -> (Char.char * stream) option
	val getpos : stream -> int
	val getlineNo : stream -> int
	val subtract : stream * stream -> string
	val eof : stream -> bool
	val lastWasNL : stream -> bool

      end = struct

        structure TIO = TextIO
        structure TSIO = TIO.StreamIO
	structure TPIO = TextPrimIO

        datatype stream = Stream of {
            strm : TSIO.instream,
	    id : int,  (* track which streams originated 
			* from the same stream *)
	    pos : int,
	    lineNo : int,
	    lastWasNL : bool
          }

	local
	  val next = ref 0
	in
	fun nextId() = !next before (next := !next + 1)
	end

	val initPos = 2 (* ml-lex bug compatibility *)

	fun mkStream inputN = let
              val strm = TSIO.mkInstream 
			   (TPIO.RD {
			        name = "lexgen",
				chunkSize = 4096,
				readVec = SOME inputN,
				readArr = NONE,
				readVecNB = NONE,
				readArrNB = NONE,
				block = NONE,
				canInput = NONE,
				avail = (fn () => NONE),
				getPos = NONE,
				setPos = NONE,
				endPos = NONE,
				verifyPos = NONE,
				close = (fn () => ()),
				ioDesc = NONE
			      }, "")
	      in 
		Stream {strm = strm, id = nextId(), pos = initPos, lineNo = 1,
			lastWasNL = true}
	      end

	fun fromStream strm = Stream {
		strm = strm, id = nextId(), pos = initPos, lineNo = 1, lastWasNL = true
	      }

	fun getc (Stream {strm, pos, id, lineNo, ...}) = (case TSIO.input1 strm
              of NONE => NONE
	       | SOME (c, strm') => 
		   SOME (c, Stream {
			        strm = strm', 
				pos = pos+1, 
				id = id,
				lineNo = lineNo + 
					 (if c = #"\n" then 1 else 0),
				lastWasNL = (c = #"\n")
			      })
	     (* end case*))

	fun getpos (Stream {pos, ...}) = pos

	fun getlineNo (Stream {lineNo, ...}) = lineNo

	fun subtract (new, old) = let
	      val Stream {strm = strm, pos = oldPos, id = oldId, ...} = old
	      val Stream {pos = newPos, id = newId, ...} = new
              val (diff, _) = if newId = oldId andalso newPos >= oldPos
			      then TSIO.inputN (strm, newPos - oldPos)
			      else raise Fail 
				"BUG: yyInput: attempted to subtract incompatible streams"
	      in 
		diff 
	      end

	fun eof s = not (isSome (getc s))

	fun lastWasNL (Stream {lastWasNL, ...}) = lastWasNL

      end

    datatype yystart_state = 
INITIAL
    structure UserDeclarations = 
      struct

structure T = Tokens
structure I = Interface
structure A = Absyn

open I

type pos = I.pos
type svalue = T.svalue
type ('a,'b) token = ('a,'b) T.token
type lexresult = (svalue,pos) token list * string

fun eof() = ([T.EOF(!line,!line)],"")

fun CN(s:string) = ([T.CN(A.ConS s,!line,!line)],s)
fun RN(s:string) = ([T.RN(A.ConR s,!line,!line)],s)
fun PN(s:string) = ([T.PN(A.ConS s,!line,!line)],s)
fun IV(s:string) = ([T.IV(A.ConS s,!line,!line)],s)
fun TV(s:string) = ([T.TV(A.ConR s,!line,!line)],s)
fun CA(s:string) = ([T.CA(A.ConS s,!line,!line)],s)
fun IA(s:string) = ([T.IA(A.ConS s,!line,!line)],s)
fun IAComp(s:string) = ([T.IAComp(A.ConR s,!line,!line)],s)
fun P(s:string) = ([T.P(A.ConR s,!line,!line)],s)



      end

    datatype yymatch 
      = yyNO_MATCH
      | yyMATCH of yyInput.stream * action * yymatch
    withtype action = yyInput.stream * yymatch -> UserDeclarations.lexresult

    local

    val yytable = 
#[
([(#"\^@",#"\b",1),
(#"\v",#"\^_",1),
(#"!",#"-",1),
(#"/",#"@",1),
(#"C",#"C",1),
(#"G",#"G",1),
(#"L",#"L",1),
(#"O",#"Q",1),
(#"T",#"V",1),
(#"X",#"`",1),
(#"c",#"c",1),
(#"f",#"f",1),
(#"h",#"h",1),
(#"m",#"m",1),
(#"o",#"r",1),
(#"u",#"u",1),
(#"w",#"\194",1),
(#"\196",#"\255",1),
(#"\t",#"\t",2),
(#" ",#" ",2),
(#"\n",#"\n",3),
(#".",#".",4),
(#"A",#"A",5),
(#"B",#"B",6),
(#"D",#"D",7),
(#"E",#"E",8),
(#"F",#"F",9),
(#"H",#"H",10),
(#"I",#"I",11),
(#"J",#"J",12),
(#"j",#"j",12),
(#"K",#"K",13),
(#"M",#"M",14),
(#"N",#"N",15),
(#"R",#"R",16),
(#"S",#"S",17),
(#"W",#"W",18),
(#"a",#"a",19),
(#"b",#"b",20),
(#"d",#"d",21),
(#"e",#"e",22),
(#"g",#"g",23),
(#"i",#"i",24),
(#"k",#"k",25),
(#"l",#"l",26),
(#"n",#"n",27),
(#"s",#"s",28),
(#"t",#"t",29),
(#"v",#"v",30),
(#"\195",#"\195",31)], [0]), ([], [39]), ([(#"\t",#"\t",206),
(#" ",#" ",206)], [0, 39]), ([], [1]), ([], [2, 39]), ([(#"l",#"l",100),
(#"n",#"n",203)], [39]), ([(#"e",#"e",195),
(#"r",#"r",196)], [39]), ([(#"i",#"i",190)], [39]), ([(#"i",#"i",185),
(#"m",#"m",186)], [39]), ([(#"r",#"r",182)], [39]), ([(#"a",#"a",169),
(#"u",#"u",170)], [39]), ([(#"t",#"t",163)], [39]), ([(#"e",#"e",160)], [39]), ([(#"e",#"e",157)], [39]), ([(#"a",#"a",147)], [39]), ([(#"a",#"a",141)], [39]), ([(#"o",#"o",139)], [39]), ([(#"c",#"c",114),
(#"p",#"p",115),
(#"t",#"t",116)], [39]), ([(#"a",#"a",110)], [39]), ([(#"l",#"l",100),
(#"r",#"r",101)], [39]), ([(#"r",#"r",94)], [39]), ([(#"a",#"a",90),
(#"e",#"e",91),
(#"i",#"i",92)], [39]), ([(#"i",#"i",86)], [39]), ([(#"e",#"e",81)], [39]), ([(#"n",#"n",78),
(#"s",#"s",79)], [39]), ([(#"e",#"e",69)], [39]), ([(#"i",#"i",64)], [39]), ([(#"e",#"e",59)], [39]), ([(#"c",#"c",48)], [39]), ([(#"o",#"o",43)], [39]), ([(#"o",#"o",41)], [39]), ([(#"\164",#"\164",32)], [39]), ([(#"l",#"l",33)], []), ([(#"t",#"t",34)], []), ([(#"e",#"e",35)], []), ([(#"r",#"r",36)], []), ([(#" ",#" ",37)], []), ([(#"a",#"a",38)], []), ([(#"l",#"l",39)], []), ([(#"s",#"s",40)], []), ([], [34]), ([(#"n",#"n",42)], []), ([], [4]), ([(#"l",#"l",44)], []), ([(#"l",#"l",45)], []), ([(#"e",#"e",46)], [30]), ([(#"r",#"r",47)], [30]), ([], [30]), ([(#"h",#"h",49)], []), ([(#"\195",#"\195",50)], []), ([(#"\182",#"\182",51)], []), ([(#"n",#"n",52)], []), ([(#"e",#"e",53)], [32]), ([(#"r",#"r",54)], []), ([(#" ",#" ",55)], []), ([(#"a",#"a",56)], []), ([(#"l",#"l",57)], []), ([(#"s",#"s",58)], []), ([], [33]), ([(#"t",#"t",60)], []), ([(#"t",#"t",61)], []), ([(#"e",#"e",62)], [31]), ([(#"n",#"n",63),
(#"r",#"r",63)], [31]), ([], [31]), ([(#"e",#"e",65)], []), ([(#"b",#"b",66)], []), ([(#"e",#"e",67),
(#"t",#"t",68)], []), ([(#"n",#"n",68)], []), ([], [38]), ([(#"i",#"i",70),
(#"n",#"n",71)], []), ([(#"n",#"n",75)], []), ([(#"n",#"n",72)], []), ([(#"e",#"e",73),
(#"t",#"t",74)], []), ([(#"n",#"n",74)], []), ([], [37]), ([(#"e",#"e",76)], [7]), ([(#"n",#"n",77),
(#"s",#"s",77)], [7]), ([], [7]), ([], [35]), ([(#"t",#"t",80)], []), ([], [3]), ([(#"s",#"s",82)], []), ([(#"u",#"u",83)], []), ([(#"n",#"n",84)], []), ([(#"d",#"d",85)], []), ([], [28]), ([(#"n",#"n",87)], []), ([(#"e",#"e",88)], [6]), ([(#"n",#"n",89),
(#"s",#"s",89)], [6]), ([], [6]), ([(#"s",#"s",93)], []), ([(#"n",#"n",93),
(#"r",#"r",93)], []), ([(#"e",#"e",93)], []), ([], [5]), ([(#"a",#"a",95)], []), ([(#"u",#"u",96)], []), ([(#"n",#"n",97)], []), ([(#"e",#"e",98)], [29]), ([(#"r",#"r",99)], [29]), ([], [29]), ([(#"l",#"l",108)], []), ([(#"b",#"b",102)], []), ([(#"e",#"e",103)], []), ([(#"i",#"i",104)], []), ([(#"t",#"t",105)], []), ([(#"e",#"e",106)], []), ([(#"n",#"n",107),
(#"t",#"t",107)], []), ([], [36]), ([(#"e",#"e",109)], []), ([], [8]), ([(#"l",#"l",111)], []), ([(#"d",#"d",112)], []), ([(#"i",#"i",113)], []), ([], [13]), ([(#"h",#"h",132)], []), ([(#"a",#"a",127)], []), ([(#"a",#"a",117),
(#"u",#"u",118)], []), ([(#"d",#"d",125)], []), ([(#"d",#"d",119)], []), ([(#"e",#"e",120)], []), ([(#"n",#"n",121)], []), ([(#"t",#"t",122)], []), ([(#"e",#"e",123)], [22]), ([(#"n",#"n",124)], []), ([], [22]), ([(#"t",#"t",126)], []), ([], [27]), ([(#"n",#"n",128)], []), ([(#"i",#"i",129)], []), ([(#"e",#"e",130)], []), ([(#"n",#"n",131)], []), ([], [17]), ([(#"w",#"w",133)], []), ([(#"e",#"e",134)], []), ([(#"s",#"s",135)], []), ([(#"t",#"t",136)], []), ([(#"e",#"e",137)], []), ([(#"r",#"r",138)], []), ([], [25]), ([(#"m",#"m",140)], []), ([], [14]), ([(#"c",#"c",142)], []), ([(#"h",#"h",143)], []), ([(#"b",#"b",144)], []), ([(#"a",#"a",145)], []), ([(#"r",#"r",146)], []), ([], [23]), ([(#"d",#"d",148),
(#"n",#"n",149),
(#"r",#"r",150)], []), ([(#"r",#"r",154)], []), ([(#"n",#"n",153)], []), ([(#"i",#"i",151)], []), ([(#"a",#"a",152)], []), ([], [12]), ([], [18]), ([(#"i",#"i",155)], []), ([(#"d",#"d",156)], []), ([], [15]), ([(#"i",#"i",158)], []), ([(#"n",#"n",159)], []), ([(#"e",#"e",77)], [7]), ([(#"d",#"d",161)], []), ([(#"e",#"e",162)], []), ([(#"n",#"n",109),
(#"r",#"r",109)], [8]), ([(#"a",#"a",164)], []), ([(#"l",#"l",165)], []), ([(#"i",#"i",166)], []), ([(#"e",#"e",167)], []), ([(#"n",#"n",168)], []), ([], [16]), ([(#"u",#"u",174)], []), ([(#"n",#"n",171)], []), ([(#"d",#"d",172)], []), ([(#"e",#"e",173)], [20]), ([], [20]), ([(#"p",#"p",175)], []), ([(#"t",#"t",176)], []), ([(#"s",#"s",177)], []), ([(#"t",#"t",178)], []), ([(#"a",#"a",179)], []), ([(#"d",#"d",180)], []), ([(#"t",#"t",181)], []), ([], [26]), ([(#"a",#"a",183)], []), ([(#"u",#"u",184)], []), ([], [19]), ([(#"n",#"n",189)], []), ([(#"i",#"i",187)], []), ([(#"l",#"l",188)], []), ([], [11]), ([(#"e",#"e",89)], [6]), ([(#"e",#"e",191)], []), ([(#"t",#"t",192)], []), ([(#"e",#"e",193)], []), ([(#"r",#"r",194)], []), ([], [10]), ([(#"r",#"r",201)], []), ([(#"u",#"u",197)], []), ([(#"d",#"d",198)], []), ([(#"e",#"e",199)], []), ([(#"r",#"r",200)], []), ([], [24]), ([(#"g",#"g",202)], []), ([], [21]), ([(#"n",#"n",204)], []), ([(#"a",#"a",205)], []), ([], [9]), ([(#"\t",#"\t",206),
(#" ",#" ",206)], [0])]

    fun mk yyins = let
        (* current start state *)
        val yyss = ref INITIAL
	fun YYBEGIN ss = (yyss := ss)
	(* current input stream *)
        val yystrm = ref yyins
	(* get one char of input *)
	val yygetc = yyInput.getc
	(* create yytext *)
	fun yymktext(strm) = yyInput.subtract (strm, !yystrm)
        open UserDeclarations
        fun lex 
(yyarg as ()) = let 
     fun continue() = let
            val yylastwasn = yyInput.lastWasNL (!yystrm)
            fun yystuck (yyNO_MATCH) = raise Fail "stuck state"
	      | yystuck (yyMATCH (strm, action, old)) = 
		  action (strm, old)
	    val yypos = yyInput.getpos (!yystrm)
	    val yygetlineNo = yyInput.getlineNo
	    fun yyactsToMatches (strm, [],	  oldMatches) = oldMatches
	      | yyactsToMatches (strm, act::acts, oldMatches) = 
		  yyMATCH (strm, act, yyactsToMatches (strm, acts, oldMatches))
	    fun yygo actTable = 
		(fn (~1, _, oldMatches) => yystuck oldMatches
		  | (curState, strm, oldMatches) => let
		      val (transitions, finals') = Vector.sub (yytable, curState)
		      val finals = map (fn i => Vector.sub (actTable, i)) finals'
		      fun tryfinal() = 
		            yystuck (yyactsToMatches (strm, finals, oldMatches))
		      fun find (c, []) = NONE
			| find (c, (c1, c2, s)::ts) = 
		            if c1 <= c andalso c <= c2 then SOME s
			    else find (c, ts)
		      in case yygetc strm
			  of SOME(c, strm') => 
			       (case find (c, transitions)
				 of NONE => tryfinal()
				  | SOME n => 
				      yygo actTable
					(n, strm', 
					 yyactsToMatches (strm, finals, oldMatches)))
			   | NONE => tryfinal()
		      end)
	    in 
let
fun yyAction0 (strm, lastMatch : yymatch) = (yystrm := strm; (lex()))
fun yyAction1 (strm, lastMatch : yymatch) = (yystrm := strm;
      (next_line(); lex()))
fun yyAction2 (strm, lastMatch : yymatch) = let
      val yytext = yymktext(strm)
      in
        yystrm := strm; ([T.PUNKT(!line,!line)],yytext)
      end
fun yyAction3 (strm, lastMatch : yymatch) = let
      val yytext = yymktext(strm)
      in
        yystrm := strm; ([T.CopV(!line,!line)],yytext)
      end
fun yyAction4 (strm, lastMatch : yymatch) = let
      val yytext = yymktext(strm)
      in
        yystrm := strm; ([T.von(!line,!line)],yytext)
      end
fun yyAction5 (strm, lastMatch : yymatch) = let
      val yytext = yymktext(strm)
      in
        yystrm := strm; ([T.Det(!line,!line)],yytext)
      end
fun yyAction6 (strm, lastMatch : yymatch) = let
      val yytext = yymktext(strm)
      in
        yystrm := strm; ([T.EQ(!line,!line)],yytext)
      end
fun yyAction7 (strm, lastMatch : yymatch) = let
      val yytext = yymktext(strm)
      in
        yystrm := strm; ([T.NQ(!line,!line)],yytext)
      end
fun yyAction8 (strm, lastMatch : yymatch) = let
      val yytext = yymktext(strm)
      in
        yystrm := strm; ([T.UQ(!line,!line)],yytext)
      end
fun yyAction9 (strm, lastMatch : yymatch) = (yystrm := strm; (PN "Anna"))
fun yyAction10 (strm, lastMatch : yymatch) = (yystrm := strm; (PN "Dieter"))
fun yyAction11 (strm, lastMatch : yymatch) = (yystrm := strm; (PN "Emil"))
fun yyAction12 (strm, lastMatch : yymatch) = (yystrm := strm; (PN "Maria"))
fun yyAction13 (strm, lastMatch : yymatch) = (yystrm := strm; (PN "Waldi"))
fun yyAction14 (strm, lastMatch : yymatch) = (yystrm := strm; (PN "Rom"))
fun yyAction15 (strm, lastMatch : yymatch) = (yystrm := strm; (PN "Madrid"))
fun yyAction16 (strm, lastMatch : yymatch) = (yystrm := strm; (PN "Italien"))
fun yyAction17 (strm, lastMatch : yymatch) = (yystrm := strm; (PN "Spanien"))
fun yyAction18 (strm, lastMatch : yymatch) = (yystrm := strm; (CN "Mann"))
fun yyAction19 (strm, lastMatch : yymatch) = (yystrm := strm; (CN "Frau"))
fun yyAction20 (strm, lastMatch : yymatch) = (yystrm := strm; (CN "Hund"))
fun yyAction21 (strm, lastMatch : yymatch) = (yystrm := strm; (CN "Berg"))
fun yyAction22 (strm, lastMatch : yymatch) = (yystrm := strm; (CN "Student"))
fun yyAction23 (strm, lastMatch : yymatch) = (yystrm := strm; (RN "Nachbar"))
fun yyAction24 (strm, lastMatch : yymatch) = (yystrm := strm; (RN "Bruder"))
fun yyAction25 (strm, lastMatch : yymatch) = (yystrm := strm; (RN "Schwester"))
fun yyAction26 (strm, lastMatch : yymatch) = (yystrm := strm;
      (RN "Hauptstadt"))
fun yyAction27 (strm, lastMatch : yymatch) = (yystrm := strm; (CN "Stadt"))
fun yyAction28 (strm, lastMatch : yymatch) = (yystrm := strm; (CA "gesund"))
fun yyAction29 (strm, lastMatch : yymatch) = (yystrm := strm; (CA "braun"))
fun yyAction30 (strm, lastMatch : yymatch) = (yystrm := strm; (CA "toll"))
fun yyAction31 (strm, lastMatch : yymatch) = (yystrm := strm; (CA "nett"))
fun yyAction32 (strm, lastMatch : yymatch) = (yystrm := strm; (IA "schön"))
fun yyAction33 (strm, lastMatch : yymatch) = (yystrm := strm;
      (IAComp "schöner als"))
fun yyAction34 (strm, lastMatch : yymatch) = (yystrm := strm;
      (IAComp "älter als"))
fun yyAction35 (strm, lastMatch : yymatch) = (yystrm := strm; (P "in"))
fun yyAction36 (strm, lastMatch : yymatch) = (yystrm := strm; (IV "arbeiten"))
fun yyAction37 (strm, lastMatch : yymatch) = (yystrm := strm; (TV "kennen"))
fun yyAction38 (strm, lastMatch : yymatch) = (yystrm := strm; (TV "lieben"))
fun yyAction39 (strm, lastMatch : yymatch) = let
      val yytext = yymktext(strm)
      in
        yystrm := strm;
        (error ("ignoring illegal character " ^ yytext,
                      !line,!line); lex())
      end
val yyactTable = Vector.fromList([yyAction0, yyAction1, yyAction2, yyAction3,
  yyAction4, yyAction5, yyAction6, yyAction7, yyAction8, yyAction9, yyAction10,
  yyAction11, yyAction12, yyAction13, yyAction14, yyAction15, yyAction16,
  yyAction17, yyAction18, yyAction19, yyAction20, yyAction21, yyAction22,
  yyAction23, yyAction24, yyAction25, yyAction26, yyAction27, yyAction28,
  yyAction29, yyAction30, yyAction31, yyAction32, yyAction33, yyAction34,
  yyAction35, yyAction36, yyAction37, yyAction38, yyAction39])
in
  if yyInput.eof(!(yystrm))
    then UserDeclarations.eof(yyarg)
    else (case (!(yyss))
       of INITIAL => yygo yyactTable (0, !(yystrm), yyNO_MATCH)
      (* end case *))
end
            end
	  in 
            continue() 	  
	    handle IO.Io{cause, ...} => raise cause
          end
        in 
          lex 
        end
    in
    fun makeLexer yyinputN = mk (yyInput.mkStream yyinputN)
    fun makeLexer' ins = mk (yyInput.mkStream ins)
    end

  end
